package com.hfkj.service.cms.impl;

import com.alibaba.fastjson.JSONObject;
import com.hfkj.common.exception.BaseException;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.DateUtil;
import com.hfkj.common.utils.RedisUtil;
import com.hfkj.dao.CmsContentMapper;
import com.hfkj.entity.CmsContent;
import com.hfkj.entity.CmsContentExample;
import com.hfkj.mqtopic.TaskTopic;
import com.hfkj.service.cms.CmsContentService;
import com.hfkj.sysenum.cms.CmsStatusEnum;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.rocketmq.client.exception.MQBrokerException;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.remoting.exception.RemotingException;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @className: CmsContentServiceImpl
 * @author: HuRui
 * @date: 2024/9/24
 **/
@Service("cmsContentService")
public class CmsContentServiceImpl implements CmsContentService {
    @Resource
    private CmsContentMapper cmsContentMapper;
    @Resource
    private RocketMQTemplate rocketMQTemplate;
    @Resource
    private RedisUtil redisUtil;
    public final static String CACHE_CMS_ONLINE = "CACHE_CMS_ONLINE";
    @Override
    public void editData(CmsContent data){
        data.setUpdateTime(new Date());
        if (data.getId() == null) {
            data.setCreateTime(new Date());
            cmsContentMapper.insert(data);

        } else {
            cmsContentMapper.updateByPrimaryKey(data);
        }

        try {
            if (data.getStartTime() != null) {
                if (DateUtil.getSecondDiff(new Date(), data.getStartTime()) >= 1) {
                    Message message = new Message(TaskTopic.TASK_TOPIC.getTopic(), TaskTopic.CMS_ONLINE.getTag(), data.getId().toString().getBytes());
                    // 定时投递，执行时间
                    message.setDeliverTimeMs(data.getStartTime().getTime());
                    SendResult sendResult = rocketMQTemplate.getProducer().send(message);
                    System.out.println("开始时间："+ JSONObject.toJSONString(sendResult));
                } else {
                    // 开始时间小于当前时间
                    // 上线
                    online(data.getId());
                }
            }

            if (data.getEndTime() != null) {
                if (DateUtil.getSecondDiff(new Date(), data.getEndTime()) >= 1) {
                    Message message = new Message(TaskTopic.TASK_TOPIC.getTopic(), TaskTopic.CMS_OFFLINE.getTag(), data.getId().toString().getBytes());
                    // 定时投递，执行时间
                    message.setDeliverTimeMs(data.getEndTime().getTime());
                    SendResult sendResult = rocketMQTemplate.getProducer().send(message);
                    System.out.println("结束时间："+ JSONObject.toJSONString(sendResult));
                } else {
                    // 下线
                    offline(data.getId());
                }
            }
        } catch (BaseException e) {
            throw ErrorHelp.genException(SysCode.System,ErrorCode.COMMON_ERROR, e.getErrorMsg());
        } catch (Exception e) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.UNKNOW_ERROR, e.getMessage() );
        }
    }

    @Override
    public void update(CmsContent data) {
        data.setUpdateTime(new Date());
        cmsContentMapper.updateByPrimaryKey(data);
    }

    @Override
    public void online(Long id) {
        // 查询内容
        CmsContent cmsContent = getDetail(id);
        if (cmsContent == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到数据");
        }
        if (!CmsStatusEnum.status1.getStatus().equals(cmsContent.getStatus())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法上线，状态错误");
        }
        cmsContent.setStatus(CmsStatusEnum.status2.getStatus());
        update(cmsContent);
        // 删除缓存
        redisUtil.hdel(CmsContentServiceImpl.CACHE_CMS_ONLINE, ""+cmsContent.getType());
    }

    @Override
    public void offline(Long id) {
        // 查询内容
        CmsContent cmsContent = getDetail(id);
        if (cmsContent == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到数据");
        }
        if (!CmsStatusEnum.status2.getStatus().equals(cmsContent.getStatus())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法下线，状态错误");
        }
        cmsContent.setStatus(CmsStatusEnum.status3.getStatus());
        update(cmsContent);

        // 删除缓存
        redisUtil.hdel(CmsContentServiceImpl.CACHE_CMS_ONLINE, ""+cmsContent.getType());
    }

    @Override
    public CmsContent getDetail(Long id) {
        return cmsContentMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<CmsContent> getList(Map<String, Object> param) {
        CmsContentExample example = new CmsContentExample();
        CmsContentExample.Criteria criteria = example.createCriteria() .andStatusNotEqualTo(CmsStatusEnum.status0.getStatus());

        if (MapUtils.getInteger(param, "type") != null) {
            criteria.andTypeEqualTo(MapUtils.getInteger(param, "type"));
        }

        if (MapUtils.getInteger(param, "status") != null) {
            criteria.andStatusEqualTo(MapUtils.getInteger(param, "status"));
        }

        if (StringUtils.isNotBlank(MapUtils.getString(param, "name"))) {
            criteria.andNameLike("%"+MapUtils.getString(param, "name")+"%");
        }

        example.setOrderByClause("update_time desc");
        return cmsContentMapper.selectByExample(example);
    }

}
